@@ -15,12 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! Ballista Core Library
-//!
-//! This crate contains the Ballista core library which is used as a dependency by the ballista,
-//! ballista-scheduler, and ballista-executor crates. Refer to <https://crates.io/crates/ballista> for
-//! general Ballista documentation.
-
+#![doc = include_str!("../README.md")]
 #![allow(unused_imports)]
 pub const BALLISTA_VERSION: &str = env!("CARGO_PKG_VERSION");
 
@@ -30,7 +25,6 @@ pub fn print_version() {
 
 pub mod client;
 pub mod config;
-pub mod datasource;
 pub mod error;
 pub mod execution_plans;
 pub mod memory_stream;
