@@ -28,11 +28,7 @@ use std::{
 use ballista::context::BallistaContext;
 use ballista::prelude::{BallistaConfig, BALLISTA_DEFAULT_SHUFFLE_PARTITIONS};
 
-use datafusion::arrow::datatypes::{DataType, Field, Schema};
-use datafusion::arrow::record_batch::RecordBatch;
-use datafusion::arrow::util::pretty;
-use datafusion::datasource::parquet::ParquetTable;
-use datafusion::datasource::{CsvFile, MemTable, TableProvider};
+use datafusion::datasource::{MemTable, TableProvider};
 use datafusion::error::{DataFusionError, Result};
 use datafusion::logical_plan::LogicalPlan;
 use datafusion::parquet::basic::Compression;
@@ -40,6 +36,20 @@ use datafusion::parquet::file::properties::WriterProperties;
 use datafusion::physical_plan::display::DisplayableExecutionPlan;
 use datafusion::physical_plan::{collect, displayable};
 use datafusion::prelude::*;
+use datafusion::{
+    arrow::datatypes::{DataType, Field, Schema},
+    datasource::file_format::{csv::CsvFormat, FileFormat},
+};
+use datafusion::{
+    arrow::record_batch::RecordBatch, datasource::file_format::parquet::ParquetFormat,
+};
+use datafusion::{
+    arrow::util::pretty,
+    datasource::{
+        listing::{ListingOptions, ListingTable},
+        object_store::local::LocalFileSystem,
+    },
+};
 
 use structopt::StructOpt;
 
@@ -81,8 +91,8 @@ struct BallistaBenchmarkOpt {
     #[structopt(short = "m", long = "mem-table")]
     mem_table: bool,
 
-    /// Number of partitions to create when using MemTable as input
-    #[structopt(short = "n", long = "partitions", default_value = "8")]
+    /// Number of partitions to process in parallel
+    #[structopt(short = "p", long = "partitions", default_value = "2")]
     partitions: usize,
 
     /// Ballista executor host
@@ -92,10 +102,6 @@ struct BallistaBenchmarkOpt {
     /// Ballista executor port
     #[structopt(long = "port")]
     port: Option<u16>,
-
-    /// Number of shuffle partitions
-    #[structopt(short, long, default_value = "2")]
-    shuffle_partitions: usize,
 }
 
 #[derive(Debug, StructOpt, Clone)]
@@ -112,9 +118,9 @@ struct DataFusionBenchmarkOpt {
     #[structopt(short = "i", long = "iterations", default_value = "3")]
     iterations: usize,
 
-    /// Number of threads to use for parallel execution
-    #[structopt(short = "c", long = "concurrency", default_value = "2")]
-    concurrency: usize,
+    /// Number of partitions to process in parallel
+    #[structopt(short = "p", long = "partitions", default_value = "2")]
+    partitions: usize,
 
     /// Batch size when reading CSV or Parquet files
     #[structopt(short = "s", long = "batch-size", default_value = "8192")]
@@ -131,10 +137,6 @@ struct DataFusionBenchmarkOpt {
     /// Load the data into a MemTable before executing the query
     #[structopt(short = "m", long = "mem-table")]
     mem_table: bool,
-
-    /// Number of partitions to create when using MemTable as input
-    #[structopt(short = "n", long = "partitions", default_value = "8")]
-    partitions: usize,
 }
 
 #[derive(Debug, StructOpt)]
@@ -152,7 +154,7 @@ struct ConvertOpt {
     file_format: String,
 
     /// Compression to use when writing Parquet files
-    #[structopt(short = "c", long = "compression", default_value = "snappy")]
+    #[structopt(short = "c", long = "compression", default_value = "zstd")]
     compression: String,
 
     /// Number of partitions to produce
@@ -203,7 +205,7 @@ async fn main() -> Result<()> {
 async fn benchmark_datafusion(opt: DataFusionBenchmarkOpt) -> Result<Vec<RecordBatch>> {
     println!("Running benchmarks with the following options: {:?}", opt);
     let config = ExecutionConfig::new()
-        .with_concurrency(opt.concurrency)
+        .with_target_partitions(opt.partitions)
         .with_batch_size(opt.batch_size);
     let mut ctx = ExecutionContext::with_config(config);
 
@@ -213,7 +215,7 @@ async fn benchmark_datafusion(opt: DataFusionBenchmarkOpt) -> Result<Vec<RecordB
             opt.path.to_str().unwrap(),
             table,
             opt.file_format.as_str(),
-            opt.concurrency,
+            opt.partitions,
         )?;
         if opt.mem_table {
             println!("Loading table '{}' into memory", table);
@@ -257,7 +259,7 @@ async fn benchmark_ballista(opt: BallistaBenchmarkOpt) -> Result<()> {
     let config = BallistaConfig::builder()
         .set(
             BALLISTA_DEFAULT_SHUFFLE_PARTITIONS,
-            &format!("{}", opt.shuffle_partitions),
+            &format!("{}", opt.partitions),
         )
         .build()
         .map_err(|e| DataFusionError::Execution(format!("{:?}", e)))?;
@@ -280,6 +282,7 @@ async fn benchmark_ballista(opt: BallistaBenchmarkOpt) -> Result<()> {
                     .has_header(false)
                     .file_extension(".tbl");
                 ctx.register_csv(table, &path, options)
+                    .await
                     .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))?;
             }
             "csv" => {
@@ -287,11 +290,13 @@ async fn benchmark_ballista(opt: BallistaBenchmarkOpt) -> Result<()> {
                 let schema = get_schema(table);
                 let options = CsvReadOptions::new().schema(&schema).has_header(true);
                 ctx.register_csv(table, &path, options)
+                    .await
                     .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))?;
             }
             "parquet" => {
                 let path = format!("{}/{}", path, table);
                 ctx.register_parquet(table, &path)
+                    .await
                     .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))?;
             }
             other => {
@@ -309,6 +314,7 @@ async fn benchmark_ballista(opt: BallistaBenchmarkOpt) -> Result<()> {
         let start = Instant::now();
         let df = ctx
             .sql(&sql)
+            .await
             .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))?;
         let batches = df
             .collect()
@@ -356,7 +362,7 @@ async fn execute_query(
     if debug {
         println!("=== Optimized logical plan ===\n{:?}\n", plan);
     }
-    let physical_plan = ctx.create_physical_plan(&plan)?;
+    let physical_plan = ctx.create_physical_plan(&plan).await?;
     if debug {
         println!(
             "=== Physical plan ===\n{}\n",
@@ -392,7 +398,7 @@ async fn convert_tbl(opt: ConvertOpt) -> Result<()> {
         let mut ctx = ExecutionContext::with_config(config);
 
         // build plan to read the TBL file
-        let mut csv = ctx.read_csv(&input_path, options)?;
+        let mut csv = ctx.read_csv(&input_path, options).await?;
 
         // optionally, repartition the file
         if opt.partitions > 1 {
@@ -402,7 +408,7 @@ async fn convert_tbl(opt: ConvertOpt) -> Result<()> {
         // create the physical plan
         let csv = csv.to_logical_plan();
         let csv = ctx.optimize(&csv)?;
-        let csv = ctx.create_physical_plan(&csv)?;
+        let csv = ctx.create_physical_plan(&csv).await?;
 
         let output_path = output_root_path.join(table);
         let output_path = output_path.to_str().unwrap().to_owned();
@@ -451,36 +457,54 @@ fn get_table(
     path: &str,
     table: &str,
     table_format: &str,
-    max_concurrency: usize,
+    target_partitions: usize,
 ) -> Result<Arc<dyn TableProvider>> {
-    match table_format {
-        // dbgen creates .tbl ('|' delimited) files without header
-        "tbl" => {
-            let path = format!("{}/{}.tbl", path, table);
-            let schema = get_schema(table);
-            let options = CsvReadOptions::new()
-                .schema(&schema)
-                .delimiter(b'|')
-                .has_header(false)
-                .file_extension(".tbl");
+    let (format, path, extension): (Arc<dyn FileFormat>, String, &'static str) =
+        match table_format {
+            // dbgen creates .tbl ('|' delimited) files without header
+            "tbl" => {
+                let path = format!("{}/{}.tbl", path, table);
 
-            Ok(Arc::new(CsvFile::try_new(&path, options)?))
-        }
-        "csv" => {
-            let path = format!("{}/{}", path, table);
-            let schema = get_schema(table);
-            let options = CsvReadOptions::new().schema(&schema).has_header(true);
+                let format = CsvFormat::default()
+                    .with_delimiter(b'|')
+                    .with_has_header(false);
 
-            Ok(Arc::new(CsvFile::try_new(&path, options)?))
-        }
-        "parquet" => {
-            let path = format!("{}/{}", path, table);
-            Ok(Arc::new(ParquetTable::try_new(&path, max_concurrency)?))
-        }
-        other => {
-            unimplemented!("Invalid file format '{}'", other);
-        }
-    }
+                (Arc::new(format), path, ".tbl")
+            }
+            "csv" => {
+                let path = format!("{}/{}", path, table);
+                let format = CsvFormat::default()
+                    .with_delimiter(b',')
+                    .with_has_header(true);
+
+                (Arc::new(format), path, ".csv")
+            }
+            "parquet" => {
+                let path = format!("{}/{}", path, table);
+                let format = ParquetFormat::default().with_enable_pruning(true);
+
+                (Arc::new(format), path, ".parquet")
+            }
+            other => {
+                unimplemented!("Invalid file format '{}'", other);
+            }
+        };
+    let schema = Arc::new(get_schema(table));
+
+    let options = ListingOptions {
+        format,
+        file_extension: extension.to_owned(),
+        target_partitions,
+        collect_stat: true,
+        table_partition_cols: vec![],
+    };
+
+    Ok(Arc::new(ListingTable::new(
+        Arc::new(LocalFileSystem {}),
+        path,
+        schema,
+        options,
+    )))
 }
 
 fn get_schema(table: &str) -> Schema {
@@ -490,7 +514,7 @@ fn get_schema(table: &str) -> Schema {
 
     match table {
         "part" => Schema::new(vec![
-            Field::new("p_partkey", DataType::UInt64, false),
+            Field::new("p_partkey", DataType::Int64, false),
             Field::new("p_name", DataType::Utf8, false),
             Field::new("p_mfgr", DataType::Utf8, false),
             Field::new("p_brand", DataType::Utf8, false),
@@ -502,28 +526,28 @@ fn get_schema(table: &str) -> Schema {
         ]),
 
         "supplier" => Schema::new(vec![
-            Field::new("s_suppkey", DataType::UInt64, false),
+            Field::new("s_suppkey", DataType::Int64, false),
             Field::new("s_name", DataType::Utf8, false),
             Field::new("s_address", DataType::Utf8, false),
-            Field::new("s_nationkey", DataType::UInt64, false),
+            Field::new("s_nationkey", DataType::Int64, false),
             Field::new("s_phone", DataType::Utf8, false),
             Field::new("s_acctbal", DataType::Float64, false),
             Field::new("s_comment", DataType::Utf8, false),
         ]),
 
         "partsupp" => Schema::new(vec![
-            Field::new("ps_partkey", DataType::UInt64, false),
-            Field::new("ps_suppkey", DataType::UInt64, false),
+            Field::new("ps_partkey", DataType::Int64, false),
+            Field::new("ps_suppkey", DataType::Int64, false),
             Field::new("ps_availqty", DataType::Int32, false),
             Field::new("ps_supplycost", DataType::Float64, false),
             Field::new("ps_comment", DataType::Utf8, false),
         ]),
 
         "customer" => Schema::new(vec![
-            Field::new("c_custkey", DataType::UInt64, false),
+            Field::new("c_custkey", DataType::Int64, false),
             Field::new("c_name", DataType::Utf8, false),
             Field::new("c_address", DataType::Utf8, false),
-            Field::new("c_nationkey", DataType::UInt64, false),
+            Field::new("c_nationkey", DataType::Int64, false),
             Field::new("c_phone", DataType::Utf8, false),
             Field::new("c_acctbal", DataType::Float64, false),
             Field::new("c_mktsegment", DataType::Utf8, false),
@@ -531,8 +555,8 @@ fn get_schema(table: &str) -> Schema {
         ]),
 
         "orders" => Schema::new(vec![
-            Field::new("o_orderkey", DataType::UInt64, false),
-            Field::new("o_custkey", DataType::UInt64, false),
+            Field::new("o_orderkey", DataType::Int64, false),
+            Field::new("o_custkey", DataType::Int64, false),
             Field::new("o_orderstatus", DataType::Utf8, false),
             Field::new("o_totalprice", DataType::Float64, false),
             Field::new("o_orderdate", DataType::Date32, false),
@@ -543,9 +567,9 @@ fn get_schema(table: &str) -> Schema {
         ]),
 
         "lineitem" => Schema::new(vec![
-            Field::new("l_orderkey", DataType::UInt64, false),
-            Field::new("l_partkey", DataType::UInt64, false),
-            Field::new("l_suppkey", DataType::UInt64, false),
+            Field::new("l_orderkey", DataType::Int64, false),
+            Field::new("l_partkey", DataType::Int64, false),
+            Field::new("l_suppkey", DataType::Int64, false),
             Field::new("l_linenumber", DataType::Int32, false),
             Field::new("l_quantity", DataType::Float64, false),
             Field::new("l_extendedprice", DataType::Float64, false),
@@ -562,14 +586,14 @@ fn get_schema(table: &str) -> Schema {
         ]),
 
         "nation" => Schema::new(vec![
-            Field::new("n_nationkey", DataType::UInt64, false),
+            Field::new("n_nationkey", DataType::Int64, false),
             Field::new("n_name", DataType::Utf8, false),
-            Field::new("n_regionkey", DataType::UInt64, false),
+            Field::new("n_regionkey", DataType::Int64, false),
             Field::new("n_comment", DataType::Utf8, false),
         ]),
 
         "region" => Schema::new(vec![
-            Field::new("r_regionkey", DataType::UInt64, false),
+            Field::new("r_regionkey", DataType::Int64, false),
             Field::new("r_name", DataType::Utf8, false),
             Field::new("r_comment", DataType::Utf8, false),
         ]),
@@ -974,7 +998,7 @@ mod tests {
         // Tests running query with empty tables, to see whether they run succesfully.
 
         let config = ExecutionConfig::new()
-            .with_concurrency(1)
+            .with_target_partitions(1)
             .with_batch_size(10);
         let mut ctx = ExecutionContext::with_config(config);
 
@@ -1004,7 +1028,9 @@ mod tests {
                 .schema(&schema)
                 .delimiter(b'|')
                 .file_extension(".out");
-            let df = ctx.read_csv(&format!("{}/answers/q{}.out", path, n), options)?;
+            let df = ctx
+                .read_csv(&format!("{}/answers/q{}.out", path, n), options)
+                .await?;
             let df = df.select(
                 get_answer_schema(n)
                     .fields()
@@ -1027,12 +1053,11 @@ mod tests {
                 query: n,
                 debug: false,
                 iterations: 1,
-                concurrency: 2,
+                partitions: 2,
                 batch_size: 8192,
                 path: PathBuf::from(path.to_string()),
                 file_format: "tbl".to_string(),
                 mem_table: false,
-                partitions: 16,
             };
             let actual = benchmark_datafusion(opt).await?;
 
@@ -1066,9 +1091,9 @@ mod tests {
         use datafusion::physical_plan::ExecutionPlan;
         use std::convert::TryInto;
 
-        fn round_trip_query(n: usize) -> Result<()> {
+        async fn round_trip_query(n: usize) -> Result<()> {
             let config = ExecutionConfig::new()
-                .with_concurrency(1)
+                .with_target_partitions(1)
                 .with_batch_size(10);
             let mut ctx = ExecutionContext::with_config(config);
 
@@ -1084,10 +1109,13 @@ mod tests {
                     .delimiter(b'|')
                     .has_header(false)
                     .file_extension(".tbl");
-                let provider = CsvFile::try_new(
-                    &format!("{}/{}.tbl", tpch_data_path, table),
-                    options,
-                )?;
+                let listing_options = options.to_listing_options(1);
+                let provider = ListingTable::new(
+                    Arc::new(LocalFileSystem {}),
+                    format!("{}/{}.tbl", tpch_data_path, table),
+                    Arc::new(schema),
+                    listing_options,
+                );
                 ctx.register_table(table, Arc::new(provider))?;
             }
 
@@ -1108,12 +1136,12 @@ mod tests {
             assert_eq!(
                 format!("{:?}", plan),
                 format!("{:?}", round_trip),
-                "opitmized logical plan round trip failed"
+                "optimized logical plan round trip failed"
             );
 
             // test physical plan roundtrip
             if env::var("TPCH_DATA").is_ok() {
-                let physical_plan = ctx.create_physical_plan(&plan)?;
+                let physical_plan = ctx.create_physical_plan(&plan).await?;
                 let proto: protobuf::PhysicalPlanNode =
                     (physical_plan.clone()).try_into().unwrap();
                 let round_trip: Arc<dyn ExecutionPlan> = (&proto).try_into().unwrap();
@@ -1129,9 +1157,9 @@ mod tests {
 
         macro_rules! test_round_trip {
             ($tn:ident, $query:expr) => {
-                #[test]
-                fn $tn() -> Result<()> {
-                    round_trip_query($query)
+                #[tokio::test]
+                async fn $tn() -> Result<()> {
+                    round_trip_query($query).await
                 }
             };
         }
@@ -1141,7 +1169,10 @@ mod tests {
         test_round_trip!(q5, 5);
         test_round_trip!(q6, 6);
         test_round_trip!(q7, 7);
+        test_round_trip!(q8, 8);
+        test_round_trip!(q9, 9);
         test_round_trip!(q10, 10);
         test_round_trip!(q12, 12);
+        test_round_trip!(q13, 13);
     }
 }
