@@ -15,25 +15,23 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#![allow(bare_trait_objects)]
-
 use clap::{crate_version, App, Arg};
 use datafusion::error::Result;
-use datafusion::execution::context::{ExecutionConfig, ExecutionContext};
+use datafusion::execution::context::ExecutionConfig;
 use datafusion_cli::{
+    context::Context,
+    exec,
     print_format::{all_print_formats, PrintFormat},
-    PrintOptions,
+    print_options::PrintOptions,
+    DATAFUSION_CLI_VERSION,
 };
-use rustyline::Editor;
 use std::env;
 use std::fs::File;
-use std::io::prelude::*;
 use std::io::BufReader;
 use std::path::Path;
-use std::time::Instant;
 
 #[tokio::main]
-pub async fn main() {
+pub async fn main() -> Result<()> {
     let matches = App::new("DataFusion")
         .version(crate_version!())
         .about(
@@ -82,6 +80,18 @@ pub async fn main() {
                 )
                 .takes_value(true),
         )
+        .arg(
+            Arg::with_name("host")
+                .help("Ballista scheduler host")
+                .long("host")
+                .takes_value(true),
+        )
+        .arg(
+            Arg::with_name("port")
+                .help("Ballista scheduler port")
+                .long("port")
+                .takes_value(true),
+        )
         .arg(
             Arg::with_name("quiet")
                 .help("Reduce printing other than the results and work quietly")
@@ -91,6 +101,17 @@ pub async fn main() {
         )
         .get_matches();
 
+    let quiet = matches.is_present("quiet");
+
+    if !quiet {
+        println!("DataFusion CLI v{}\n", DATAFUSION_CLI_VERSION);
+    }
+
+    let host = matches.value_of("host");
+    let port = matches
+        .value_of("port")
+        .and_then(|port| port.parse::<u16>().ok());
+
     if let Some(path) = matches.value_of("data-path") {
         let p = Path::new(path);
         env::set_current_dir(&p).unwrap();
@@ -105,104 +126,32 @@ pub async fn main() {
         execution_config = execution_config.with_batch_size(batch_size);
     };
 
+    let mut ctx: Context = match (host, port) {
+        (Some(h), Some(p)) => Context::new_remote(h, p)?,
+        _ => Context::new_local(&execution_config),
+    };
+
     let format = matches
         .value_of("format")
         .expect("No format is specified")
         .parse::<PrintFormat>()
         .expect("Invalid format");
 
-    let quiet = matches.is_present("quiet");
     let print_options = PrintOptions { format, quiet };
 
     if let Some(file_paths) = matches.values_of("file") {
         let files = file_paths
             .map(|file_path| File::open(file_path).unwrap())
             .collect::<Vec<_>>();
-        let mut ctx = ExecutionContext::with_config(execution_config);
         for file in files {
             let mut reader = BufReader::new(file);
-            exec_from_lines(&mut ctx, &mut reader, print_options.clone()).await;
+            exec::exec_from_lines(&mut ctx, &mut reader, &print_options).await;
         }
     } else {
-        exec_from_repl(execution_config, print_options).await;
-    }
-}
-
-async fn exec_from_lines(
-    ctx: &mut ExecutionContext,
-    reader: &mut BufReader<File>,
-    print_options: PrintOptions,
-) {
-    let mut query = "".to_owned();
-
-    for line in reader.lines() {
-        match line {
-            Ok(line) if line.starts_with("--") => {
-                continue;
-            }
-            Ok(line) => {
-                let line = line.trim_end();
-                query.push_str(line);
-                if line.ends_with(';') {
-                    match exec_and_print(ctx, print_options.clone(), query).await {
-                        Ok(_) => {}
-                        Err(err) => println!("{:?}", err),
-                    }
-                    query = "".to_owned();
-                } else {
-                    query.push('\n');
-                }
-            }
-            _ => {
-                break;
-            }
-        }
+        exec::exec_from_repl(&mut ctx, &print_options).await;
     }
 
-    // run the left over query if the last statement doesn't contain ‘;’
-    if !query.is_empty() {
-        match exec_and_print(ctx, print_options, query).await {
-            Ok(_) => {}
-            Err(err) => println!("{:?}", err),
-        }
-    }
-}
-
-async fn exec_from_repl(execution_config: ExecutionConfig, print_options: PrintOptions) {
-    let mut ctx = ExecutionContext::with_config(execution_config);
-
-    let mut rl = Editor::<()>::new();
-    rl.load_history(".history").ok();
-
-    let mut query = "".to_owned();
-    loop {
-        match rl.readline("> ") {
-            Ok(ref line) if is_exit_command(line) && query.is_empty() => {
-                break;
-            }
-            Ok(ref line) if line.starts_with("--") => {
-                continue;
-            }
-            Ok(ref line) if line.trim_end().ends_with(';') => {
-                query.push_str(line.trim_end());
-                rl.add_history_entry(query.clone());
-                match exec_and_print(&mut ctx, print_options.clone(), query).await {
-                    Ok(_) => {}
-                    Err(err) => println!("{:?}", err),
-                }
-                query = "".to_owned();
-            }
-            Ok(ref line) => {
-                query.push_str(line);
-                query.push('\n');
-            }
-            Err(_) => {
-                break;
-            }
-        }
-    }
-
-    rl.save_history(".history").ok();
+    Ok(())
 }
 
 fn is_valid_file(dir: String) -> std::result::Result<(), String> {
@@ -227,21 +176,3 @@ fn is_valid_batch_size(size: String) -> std::result::Result<(), String> {
         _ => Err(format!("Invalid batch size '{}'", size)),
     }
 }
-
-fn is_exit_command(line: &str) -> bool {
-    let line = line.trim_end().to_lowercase();
-    line == "quit" || line == "exit"
-}
-
-async fn exec_and_print(
-    ctx: &mut ExecutionContext,
-    print_options: PrintOptions,
-    sql: String,
-) -> Result<()> {
-    let df = ctx.sql(&sql)?;
-    let now = Instant::now();
-    let results = df.collect().await?;
-
-    print_options.print_batches(&results, now)?;
-    Ok(())
-}
