@@ -17,6 +17,8 @@
 
 //! Defines the sort preserving merge plan
 
+use super::common::AbortOnDropMany;
+use super::metrics::{BaselineMetrics, ExecutionPlanMetricsSet, MetricsSet};
 use std::any::Any;
 use std::cmp::Ordering;
 use std::collections::VecDeque;
@@ -42,7 +44,7 @@ use crate::error::{DataFusionError, Result};
 use crate::physical_plan::{
     common::spawn_execution, expressions::PhysicalSortExpr, DisplayFormatType,
     Distribution, ExecutionPlan, Partitioning, PhysicalExpr, RecordBatchStream,
-    SendableRecordBatchStream,
+    SendableRecordBatchStream, Statistics,
 };
 
 /// Sort preserving merge execution plan
@@ -59,6 +61,8 @@ pub struct SortPreservingMergeExec {
     expr: Vec<PhysicalSortExpr>,
     /// The target size of yielded batches
     target_batch_size: usize,
+    /// Execution metrics
+    metrics: ExecutionPlanMetricsSet,
 }
 
 impl SortPreservingMergeExec {
@@ -72,6 +76,7 @@ impl SortPreservingMergeExec {
             input,
             expr,
             target_batch_size,
+            metrics: ExecutionPlanMetricsSet::new(),
         }
     }
 
@@ -134,6 +139,8 @@ impl ExecutionPlan for SortPreservingMergeExec {
             )));
         }
 
+        let baseline_metrics = BaselineMetrics::new(&self.metrics, partition);
+
         let input_partitions = self.input.output_partitioning().partition_count();
         match input_partitions {
             0 => Err(DataFusionError::Internal(
@@ -141,24 +148,27 @@ impl ExecutionPlan for SortPreservingMergeExec {
                     .to_owned(),
             )),
             1 => {
-                // bypass if there is only one partition to merge
+                // bypass if there is only one partition to merge (no metrics in this case either)
                 self.input.execute(0).await
             }
             _ => {
-                let streams = (0..input_partitions)
+                let (receivers, join_handles) = (0..input_partitions)
                     .into_iter()
                     .map(|part_i| {
                         let (sender, receiver) = mpsc::channel(1);
-                        spawn_execution(self.input.clone(), sender, part_i);
-                        receiver
+                        let join_handle =
+                            spawn_execution(self.input.clone(), sender, part_i);
+                        (receiver, join_handle)
                     })
-                    .collect();
+                    .unzip();
 
                 Ok(Box::pin(SortPreservingMergeStream::new(
-                    streams,
+                    receivers,
+                    AbortOnDropMany(join_handles),
                     self.schema(),
                     &self.expr,
                     self.target_batch_size,
+                    baseline_metrics,
                 )))
             }
         }
@@ -176,6 +186,14 @@ impl ExecutionPlan for SortPreservingMergeExec {
             }
         }
     }
+
+    fn metrics(&self) -> Option<MetricsSet> {
+        Some(self.metrics.clone_inner())
+    }
+
+    fn statistics(&self) -> Statistics {
+        self.input.statistics()
+    }
 }
 
 /// A `SortKeyCursor` is created from a `RecordBatch`, and a set of
@@ -323,21 +341,34 @@ struct RowIndex {
 struct SortPreservingMergeStream {
     /// The schema of the RecordBatches yielded by this stream
     schema: SchemaRef,
+
     /// The sorted input streams to merge together
-    streams: Vec<mpsc::Receiver<ArrowResult<RecordBatch>>>,
+    receivers: Vec<mpsc::Receiver<ArrowResult<RecordBatch>>>,
+
+    /// Drop helper for tasks feeding the [`receivers`](Self::receivers)
+    drop_helper: AbortOnDropMany<()>,
+
     /// For each input stream maintain a dequeue of SortKeyCursor
     ///
     /// Exhausted cursors will be popped off the front once all
     /// their rows have been yielded to the output
     cursors: Vec<VecDeque<SortKeyCursor>>,
+
     /// The accumulated row indexes for the next record batch
     in_progress: Vec<RowIndex>,
+
     /// The physical expressions to sort by
     column_expressions: Vec<Arc<dyn PhysicalExpr>>,
+
     /// The sort options for each expression
     sort_options: Vec<SortOptions>,
+
     /// The desired RecordBatch size to yield
     target_batch_size: usize,
+
+    /// used to record execution metrics
+    baseline_metrics: BaselineMetrics,
+
     /// If the stream has encountered an error
     aborted: bool,
 
@@ -347,12 +378,14 @@ struct SortPreservingMergeStream {
 
 impl SortPreservingMergeStream {
     fn new(
-        streams: Vec<mpsc::Receiver<ArrowResult<RecordBatch>>>,
+        receivers: Vec<mpsc::Receiver<ArrowResult<RecordBatch>>>,
+        drop_helper: AbortOnDropMany<()>,
         schema: SchemaRef,
         expressions: &[PhysicalSortExpr],
         target_batch_size: usize,
+        baseline_metrics: BaselineMetrics,
     ) -> Self {
-        let cursors = (0..streams.len())
+        let cursors = (0..receivers.len())
             .into_iter()
             .map(|_| VecDeque::new())
             .collect();
@@ -360,10 +393,12 @@ impl SortPreservingMergeStream {
         Self {
             schema,
             cursors,
-            streams,
+            receivers,
+            drop_helper,
             column_expressions: expressions.iter().map(|x| x.expr.clone()).collect(),
             sort_options: expressions.iter().map(|x| x.options).collect(),
             target_batch_size,
+            baseline_metrics,
             aborted: false,
             in_progress: vec![],
             next_batch_index: 0,
@@ -385,12 +420,12 @@ impl SortPreservingMergeStream {
             }
         }
 
-        let stream = &mut self.streams[idx];
+        let stream = &mut self.receivers[idx];
         if stream.is_terminated() {
             return Poll::Ready(Ok(()));
         }
 
-        // Fetch a new record and create a cursor from it
+        // Fetch a new input record and create a cursor from it
         match futures::ready!(stream.poll_next_unpin(cx)) {
             None => return Poll::Ready(Ok(())),
             Some(Err(e)) => {
@@ -539,6 +574,17 @@ impl Stream for SortPreservingMergeStream {
         mut self: Pin<&mut Self>,
         cx: &mut Context<'_>,
     ) -> Poll<Option<Self::Item>> {
+        let poll = self.poll_next_inner(cx);
+        self.baseline_metrics.record_poll(poll)
+    }
+}
+
+impl SortPreservingMergeStream {
+    #[inline]
+    fn poll_next_inner(
+        self: &mut Pin<&mut Self>,
+        cx: &mut Context<'_>,
+    ) -> Poll<Option<ArrowResult<RecordBatch>>> {
         if self.aborted {
             return Poll::Ready(None);
         }
@@ -556,6 +602,11 @@ impl Stream for SortPreservingMergeStream {
         }
 
         loop {
+            // NB timer records time taken on drop, so there are no
+            // calls to `timer.done()` below.
+            let elapsed_compute = self.baseline_metrics.elapsed_compute().clone();
+            let _timer = elapsed_compute.timer();
+
             let stream_idx = match self.next_stream_idx() {
                 Ok(Some(idx)) => idx,
                 Ok(None) if self.in_progress.is_empty() => return Poll::Ready(None),
@@ -607,21 +658,24 @@ impl RecordBatchStream for SortPreservingMergeStream {
 
 #[cfg(test)]
 mod tests {
+    use crate::datasource::object_store::local::LocalFileSystem;
+    use crate::physical_plan::metrics::MetricValue;
+    use crate::test::exec::{assert_strong_count_converges_to_zero, BlockingExec};
     use std::iter::FromIterator;
 
     use crate::arrow::array::{Int32Array, StringArray, TimestampNanosecondArray};
     use crate::assert_batches_eq;
-    use crate::datasource::CsvReadOptions;
     use crate::physical_plan::coalesce_partitions::CoalescePartitionsExec;
-    use crate::physical_plan::csv::CsvExec;
     use crate::physical_plan::expressions::col;
+    use crate::physical_plan::file_format::{CsvExec, PhysicalPlanConfig};
     use crate::physical_plan::memory::MemoryExec;
     use crate::physical_plan::sort::SortExec;
     use crate::physical_plan::{collect, common};
-    use crate::test;
+    use crate::test::{self, assert_is_pending};
 
     use super::*;
-    use futures::SinkExt;
+    use arrow::datatypes::{DataType, Field, Schema};
+    use futures::{FutureExt, SinkExt};
     use tokio_stream::StreamExt;
 
     #[tokio::test]
@@ -878,18 +932,23 @@ mod tests {
     async fn test_partition_sort() {
         let schema = test::aggr_test_schema();
         let partitions = 4;
-        let path =
+        let (_, files) =
             test::create_partitioned_csv("aggregate_test_100.csv", partitions).unwrap();
-        let csv = Arc::new(
-            CsvExec::try_new(
-                &path,
-                CsvReadOptions::new().schema(&schema),
-                None,
-                1024,
-                None,
-            )
-            .unwrap(),
-        );
+
+        let csv = Arc::new(CsvExec::new(
+            PhysicalPlanConfig {
+                object_store: Arc::new(LocalFileSystem {}),
+                file_schema: Arc::clone(&schema),
+                file_groups: files,
+                statistics: Statistics::default(),
+                projection: None,
+                batch_size: 1024,
+                limit: None,
+                table_partition_cols: vec![],
+            },
+            true,
+            b',',
+        ));
 
         let sort = vec![
             PhysicalSortExpr {
@@ -907,6 +966,10 @@ mod tests {
                 expr: col("c7", &schema).unwrap(),
                 options: SortOptions::default(),
             },
+            PhysicalSortExpr {
+                expr: col("c12", &schema).unwrap(),
+                options: SortOptions::default(),
+            },
         ];
 
         let basic = basic_sort(csv.clone(), sort.clone()).await;
@@ -915,7 +978,11 @@ mod tests {
         let basic = arrow::util::pretty::pretty_format_batches(&[basic]).unwrap();
         let partition = arrow::util::pretty::pretty_format_batches(&[partition]).unwrap();
 
-        assert_eq!(basic, partition);
+        assert_eq!(
+            basic, partition,
+            "basic:\n\n{}\n\npartition:\n\n{}\n\n",
+            basic, partition
+        );
     }
 
     // Split the provided record batch into multiple batch_size record batches
@@ -948,18 +1015,23 @@ mod tests {
     ) -> Arc<dyn ExecutionPlan> {
         let schema = test::aggr_test_schema();
         let partitions = 4;
-        let path =
+        let (_, files) =
             test::create_partitioned_csv("aggregate_test_100.csv", partitions).unwrap();
-        let csv = Arc::new(
-            CsvExec::try_new(
-                &path,
-                CsvReadOptions::new().schema(&schema),
-                None,
-                1024,
-                None,
-            )
-            .unwrap(),
-        );
+
+        let csv = Arc::new(CsvExec::new(
+            PhysicalPlanConfig {
+                object_store: Arc::new(LocalFileSystem {}),
+                file_schema: schema,
+                file_groups: files,
+                statistics: Statistics::default(),
+                projection: None,
+                batch_size: 1024,
+                limit: None,
+                table_partition_cols: vec![],
+            },
+            true,
+            b',',
+        ));
 
         let sorted = basic_sort(csv, sort).await;
         let split: Vec<_> = sizes.iter().map(|x| split_batch(&sorted, *x)).collect();
@@ -1125,42 +1197,48 @@ mod tests {
     async fn test_async() {
         let schema = test::aggr_test_schema();
         let sort = vec![PhysicalSortExpr {
-            expr: col("c7", &schema).unwrap(),
+            expr: col("c12", &schema).unwrap(),
             options: SortOptions::default(),
         }];
 
         let batches = sorted_partitioned_input(sort.clone(), &[5, 7, 3]).await;
 
         let partition_count = batches.output_partitioning().partition_count();
-        let mut tasks = Vec::with_capacity(partition_count);
-        let mut streams = Vec::with_capacity(partition_count);
+        let mut join_handles = Vec::with_capacity(partition_count);
+        let mut receivers = Vec::with_capacity(partition_count);
 
         for partition in 0..partition_count {
             let (mut sender, receiver) = mpsc::channel(1);
             let mut stream = batches.execute(partition).await.unwrap();
-            let task = tokio::spawn(async move {
+            let join_handle = tokio::spawn(async move {
                 while let Some(batch) = stream.next().await {
                     sender.send(batch).await.unwrap();
                     // This causes the MergeStream to wait for more input
                     tokio::time::sleep(tokio::time::Duration::from_millis(10)).await;
                 }
             });
-            tasks.push(task);
-            streams.push(receiver);
+            join_handles.push(join_handle);
+            receivers.push(receiver);
         }
 
+        let metrics = ExecutionPlanMetricsSet::new();
+        let baseline_metrics = BaselineMetrics::new(&metrics, 0);
+
         let merge_stream = SortPreservingMergeStream::new(
-            streams,
+            receivers,
+            // Use empty vector since we want to use the join handles ourselves
+            AbortOnDropMany(vec![]),
             batches.schema(),
             sort.as_slice(),
             1024,
+            baseline_metrics,
         );
 
         let mut merged = common::collect(Box::pin(merge_stream)).await.unwrap();
 
         // Propagate any errors
-        for task in tasks {
-            task.await.unwrap();
+        for join_handle in join_handles {
+            join_handle.await.unwrap();
         }
 
         assert_eq!(merged.len(), 1);
@@ -1170,6 +1248,91 @@ mod tests {
         let basic = arrow::util::pretty::pretty_format_batches(&[basic]).unwrap();
         let partition = arrow::util::pretty::pretty_format_batches(&[merged]).unwrap();
 
-        assert_eq!(basic, partition);
+        assert_eq!(
+            basic, partition,
+            "basic:\n\n{}\n\npartition:\n\n{}\n\n",
+            basic, partition
+        );
+    }
+
+    #[tokio::test]
+    async fn test_merge_metrics() {
+        let a: ArrayRef = Arc::new(Int32Array::from(vec![1, 2]));
+        let b: ArrayRef = Arc::new(StringArray::from_iter(vec![Some("a"), Some("c")]));
+        let b1 = RecordBatch::try_from_iter(vec![("a", a), ("b", b)]).unwrap();
+
+        let a: ArrayRef = Arc::new(Int32Array::from(vec![10, 20]));
+        let b: ArrayRef = Arc::new(StringArray::from_iter(vec![Some("b"), Some("d")]));
+        let b2 = RecordBatch::try_from_iter(vec![("a", a), ("b", b)]).unwrap();
+
+        let schema = b1.schema();
+        let sort = vec![PhysicalSortExpr {
+            expr: col("b", &schema).unwrap(),
+            options: Default::default(),
+        }];
+        let exec = MemoryExec::try_new(&[vec![b1], vec![b2]], schema, None).unwrap();
+        let merge = Arc::new(SortPreservingMergeExec::new(sort, Arc::new(exec), 1024));
+
+        let collected = collect(merge.clone()).await.unwrap();
+        let expected = vec![
+            "+----+---+",
+            "| a  | b |",
+            "+----+---+",
+            "| 1  | a |",
+            "| 10 | b |",
+            "| 2  | c |",
+            "| 20 | d |",
+            "+----+---+",
+        ];
+        assert_batches_eq!(expected, collected.as_slice());
+
+        // Now, validate metrics
+        let metrics = merge.metrics().unwrap();
+
+        assert_eq!(metrics.output_rows().unwrap(), 4);
+        assert!(metrics.elapsed_compute().unwrap() > 0);
+
+        let mut saw_start = false;
+        let mut saw_end = false;
+        metrics.iter().for_each(|m| match m.value() {
+            MetricValue::StartTimestamp(ts) => {
+                saw_start = true;
+                assert!(ts.value().unwrap().timestamp_nanos() > 0);
+            }
+            MetricValue::EndTimestamp(ts) => {
+                saw_end = true;
+                assert!(ts.value().unwrap().timestamp_nanos() > 0);
+            }
+            _ => {}
+        });
+
+        assert!(saw_start);
+        assert!(saw_end);
+    }
+
+    #[tokio::test]
+    async fn test_drop_cancel() -> Result<()> {
+        let schema =
+            Arc::new(Schema::new(vec![Field::new("a", DataType::Float32, true)]));
+
+        let blocking_exec = Arc::new(BlockingExec::new(Arc::clone(&schema), 2));
+        let refs = blocking_exec.refs();
+        let sort_preserving_merge_exec = Arc::new(SortPreservingMergeExec::new(
+            vec![PhysicalSortExpr {
+                expr: col("a", &schema)?,
+                options: SortOptions::default(),
+            }],
+            blocking_exec,
+            1,
+        ));
+
+        let fut = collect(sort_preserving_merge_exec);
+        let mut fut = fut.boxed();
+
+        assert_is_pending(&mut fut);
+        drop(fut);
+        assert_strong_count_converges_to_zero(refs).await;
+
+        Ok(())
     }
 }
